/* On the desktop, this slides in from the right.
* https://www.w3schools.com/howto/howto_js_sidenav.asp
*/
@import "~@synerty/peek-plugin-base-js/src/styles/_mixins";
@import "~@synerty/peek-plugin-base-js/src/styles/_variables";

//.ant-card-body {
//    padding: 8px;
//}

$margin: 8px;

.buttons {
    margin-top: $margin;
    display: flex;
}

.dynamic-buttons > button {
    margin-right: $margin;
}

.child-menu-items {
    height: 400px;
    overflow-y: scroll;
}
