@import "~@synerty/peek-plugin-base-js/src/styles/variables";

.margin {
    margin-top: $general_margin_1;
    margin-bottom: $general_margin_1;
}

nz-select {
    width: 100%;
}

input {
    width: 100%;
}

[nz-button] {
    margin-right: 8px;
    margin-bottom: 12px;
}

[nz-button]:first-child {
    margin-left: 8px;
    margin-right: 8px;
    margin-bottom: 12px;
}
